<?php

declare(strict_types=1);

namespace spec\Dedi\SyliusImageOptimizerPlugin\Media\Http\Controller;

use Dedi\SyliusImageOptimizerPlugin\Media\Http\Controller\ImageResolverInterface;
use Liip\ImagineBundle\Binary\BinaryInterface;
use Liip\ImagineBundle\Imagine\Cache\CacheManager;
use Liip\ImagineBundle\Imagine\Data\DataManager;
use Liip\ImagineBundle\Imagine\Filter\FilterManager;
use PhpSpec\ObjectBehavior;
use Prophecy\Argument;
use Symfony\Component\HttpFoundation\ParameterBag;
use Symfony\Component\HttpFoundation\Request;

class ImageResolverOnTheFlySpec extends ObjectBehavior
{
    function let(
        CacheManager $cacheManager,
        DataManager $dataManager,
        FilterManager $filterManager
    ) {
        $this->beConstructedWith(
            $cacheManager,
            $dataManager,
            $filterManager,
            'thumbnail',
            [
                'mode' => 'outbound',
                'allow_upscale' => false,
            ],
            'size'
        );
    }

    function it_is_initializable()
    {
        $this->shouldImplement(ImageResolverInterface::class);
    }

    function it_is_not_store_if_image_found(
        CacheManager $cacheManager,
        DataManager $dataManager,
        FilterManager $filterManager,
        Request $request,
        ParameterBag $parameterBag
    ) {

        $request->query = $parameterBag;

        $parameterBag->get('width')->shouldBeCalled()->willReturn(12);
        $parameterBag->get('height')->shouldBeCalled()->willReturn(12);
        $parameterBag->get('imageName')->shouldBeCalled()->willReturn('bar.png');
        $parameterBag->get('filters')->shouldBeCalled()->willReturn(null);

        $cacheManager->isStored(Argument::any(),Argument::any())->shouldBeCalled()->willReturn(true);
        $cacheManager->resolve(Argument::any(),Argument::any())->shouldBeCalled()->willReturn('http://localhost');

        $this->filter($request, 'foo');

        $dataManager->find()->shouldNotBeCalled();
        $filterManager->applyFilter()->shouldNotBeCalled();
        $cacheManager->store()->shouldNotBeCalled();

        $cacheManager->resolve('bar_thumbnail_outbound_0_12_12', 'foo')->shouldBeCalled();
    }

    function it_is_store_if_image_not_found(
        CacheManager $cacheManager,
        DataManager $dataManager,
        FilterManager $filterManager,
        BinaryInterface $filteredBinary,
        BinaryInterface $binary,
        Request $request,
        ParameterBag $parameterBag
    ) {

        $request->query = $parameterBag;

        $parameterBag->get('width')->shouldBeCalled()->willReturn(12);
        $parameterBag->get('height')->shouldBeCalled()->willReturn(12);
        $parameterBag->get('imageName')->shouldBeCalled()->willReturn('bar.png');
        $parameterBag->get('filters')->shouldBeCalled()->willReturn(null);

        $dataManager->find('foo','bar.png')->shouldBeCalled()->willReturn($binary);

        $filterManager->applyFilter($binary, 'foo', [
            'filters' => [
                'thumbnail' => [
                    'size' => [12, 12],
                    'mode' => 'outbound',
                    'allow_upscale' => false,
                ],
            ],
        ])->shouldBeCalled()->willReturn($filteredBinary);

        $cacheManager->resolve('bar_thumbnail_outbound_0_12_12', 'foo')->shouldBeCalled()->willReturn('http://localhost');

        $this->filter($request, 'foo');

        $cacheManager->isStored('bar_thumbnail_outbound_0_12_12', 'foo')->shouldHaveBeenCalled()->willReturn(false);
        $cacheManager->store($filteredBinary, 'bar_thumbnail_outbound_0_12_12', 'foo')->shouldBeCalled();
    }

    function it_is_not_store_if_image_found_with_folder(
        CacheManager $cacheManager,
        DataManager $dataManager,
        FilterManager $filterManager,
        Request $request,
        ParameterBag $parameterBag
    ) {

        $request->query = $parameterBag;

        $parameterBag->get('width')->shouldBeCalled()->willReturn(12);
        $parameterBag->get('height')->shouldBeCalled()->willReturn(12);
        $parameterBag->get('imageName')->shouldBeCalled()->willReturn('my/path/to/file/rc/1/bar.png');
        $parameterBag->get('filters')->shouldBeCalled()->willReturn(null);

        $cacheManager->isStored(Argument::any(),Argument::any())->shouldBeCalled()->willReturn(true);
        $cacheManager->resolve(Argument::any(),Argument::any())->shouldBeCalled()->willReturn('http://localhost');

        $this->filter($request, 'foo');

        $dataManager->find()->shouldNotBeCalled();
        $filterManager->applyFilter()->shouldNotBeCalled();
        $cacheManager->store()->shouldNotBeCalled();

        $cacheManager->resolve('my/path/to/file/rc/1/bar_thumbnail_outbound_0_12_12', 'foo')->shouldBeCalled();
    }

    function it_is_store_if_image_not_found_with_folder(
        CacheManager $cacheManager,
        DataManager $dataManager,
        FilterManager $filterManager,
        BinaryInterface $filteredBinary,
        BinaryInterface $binary,
        Request $request,
        ParameterBag $parameterBag
    ) {
        $request->query = $parameterBag;

        $parameterBag->get('width')->shouldBeCalled()->willReturn(12);
        $parameterBag->get('height')->shouldBeCalled()->willReturn(12);
        $parameterBag->get('imageName')->shouldBeCalled()->willReturn('my/path/to/file/rc/1/bar.png');
        $parameterBag->get('filters')->shouldBeCalled()->willReturn(null);

        $dataManager->find('foo','my/path/to/file/rc/1/bar.png')->shouldBeCalled()->willReturn($binary);

        $filterManager->applyFilter($binary, 'foo', [
            'filters' => [
                'thumbnail' => [
                    'size' => [12, 12],
                    'mode' => 'outbound',
                    'allow_upscale' => false,
                ],
            ],
        ])->shouldBeCalled()->willReturn($filteredBinary);

        $cacheManager->resolve('my/path/to/file/rc/1/bar_thumbnail_outbound_0_12_12', 'foo')->shouldBeCalled()->willReturn('http://localhost');

        $this->filter($request, 'foo');

        $cacheManager->isStored('my/path/to/file/rc/1/bar_thumbnail_outbound_0_12_12', 'foo')->shouldHaveBeenCalled()->willReturn(false);
        $cacheManager->store($filteredBinary, 'my/path/to/file/rc/1/bar_thumbnail_outbound_0_12_12', 'foo')->shouldBeCalled();
    }
}
