<?php

declare(strict_types=1);

namespace Dedi\SyliusImageOptimizerPlugin\DependencyInjection;

use Symfony\Component\Config\Definition\Builder\TreeBuilder;
use Symfony\Component\Config\Definition\ConfigurationInterface;

final class Configuration implements ConfigurationInterface
{
    /**
     * @inheritdoc
     */
    public function getConfigTreeBuilder(): TreeBuilder
    {
        $treeBuilder = new TreeBuilder('dedi_sylius_image_optimizer_plugin');
        if (\method_exists($treeBuilder, 'getRootNode')) {
            $rootNode = $treeBuilder->getRootNode();
        } else {
            // BC layer for symfony/config 4.1 and older
            $rootNode = $treeBuilder->root('dedi_sylius_image_optimizer_plugin');
        }

        $rootNode
            ->children()
                ->arrayNode('default')
                ->addDefaultsIfNotSet()
                    ->children()
                        ->scalarNode('filter_name')
                            ->defaultValue('thumbnail')
                        ->end()
                        ->arrayNode('filter_options')
                            ->prototype('scalar')->end()
                            ->defaultValue([
                                'mode' => 'outbound',
                                'allow_upscale' => false,
                            ])
                        ->end()
                        ->scalarNode('filter_size_keyname')
                            ->defaultValue('size')
                        ->end()
                    ->end()
                ->end()
            ->end();

        return $treeBuilder;
    }
}
