<?php

declare(strict_types=1);

namespace Dedi\SyliusImageOptimizerPlugin\Media\Http\Controller;

use Liip\ImagineBundle\Imagine\Cache\CacheManager;
use Liip\ImagineBundle\Imagine\Data\DataManager;
use Liip\ImagineBundle\Imagine\Filter\FilterManager;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;

class ImageResolverOnTheFly implements ImageResolverInterface
{
    private $cacheManager;
    private $dataManager;
    private $filterManager;
    private $defaultFilterName;
    private $defaultFilterOptions;
    private $defaultFilterSizeKeyname;

    public function __construct(
        CacheManager $cacheManager,
        DataManager $dataManager,
        FilterManager $filterManager,
        string $defaultFilterName,
        array $defaultFilterOptions,
        string $defaultFilterSizeKeyname
    ) {
        $this->cacheManager = $cacheManager;
        $this->dataManager = $dataManager;
        $this->filterManager = $filterManager;
        $this->defaultFilterName = $defaultFilterName;
        $this->defaultFilterOptions = $defaultFilterOptions;
        $this->defaultFilterSizeKeyname = $defaultFilterSizeKeyname;
    }

    public function filter(Request $request, string $filterName): Response
    {
        $queryParamterBag = $request->query;

        $width = $queryParamterBag->get('width');
        $height = $queryParamterBag->get('height');
        $imageName = $queryParamterBag->get('imageName');
        $filters = $queryParamterBag->get('filters');

        $dirname = pathinfo($imageName, \PATHINFO_DIRNAME);
        $filename = pathinfo($imageName, \PATHINFO_FILENAME);

        $filtersOption = [];
        if ($filters) {
            $filtersOption[$filters['name']] = $filters['options'];
            $fullfileName = sprintf('%s_%s%s', $filename, $filters['name'], $this->getOptionValue($filters['options']));
        } else {
            $filtersOption[$this->defaultFilterName] = array_merge([
                $this->defaultFilterSizeKeyname => [$width, $height],
            ], $this->defaultFilterOptions);

            $fullfileName = sprintf('%s_%s%s_%d_%d', $filename, $this->defaultFilterName, $this->getOptionValue($this->defaultFilterOptions), $width, $height);
        }

        $cacheName = $dirname !== '.' ? $dirname . \DIRECTORY_SEPARATOR . $fullfileName : $fullfileName;

        if (!$this->cacheManager->isStored($cacheName, $filterName)) {
            $binary = $this->dataManager->find($filterName, $imageName);

            $filteredBinary = $this->filterManager->applyFilter($binary, $filterName, [
                'filters' => $filtersOption,
            ]);

            $this->cacheManager->store($filteredBinary, $cacheName, $filterName);
        }

        return new RedirectResponse($this->cacheManager->resolve($cacheName, $filterName), Response::HTTP_MOVED_PERMANENTLY);
    }

    /**
     * Concate values of array recursively separate with underscore
     */
    private function getOptionValue(array $options): string
    {
        $valueName = '';

        foreach ($options as $value) {
            if (is_array($value)) {
                $valueName .= $this->getOptionValue($value);
            } else {
                $valueName .= '_' . (is_bool($value) ? (int) (string) $value : $value);
            }
        }

        return $valueName;
    }
}
